package com.ruoyi.sim.service.impl;

import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import static com.ruoyi.sim.service.impl.CommBuildService.CommConst.*;

@Service
public class CommBuildService {

    interface CommConst {

        /**
         * 报文长度
         */
        int MSG_LENGTH = 20;

        String PREFIX = "AA";

        String SUFFIX = "55";

        /**
         * orientation
         */
        String ORN_SEND = "01";

        /**
         * orientation
         */
        String ORN_RECEIVE = "02";

        String CMD_DATA_PLACE_HOLDER = "00000000";

        /**
         * 故障下发
         */
        String CMD_SET_FAULT = "01";
        /**
         * 故障清清除
         */
        String CMD_CLEAR_FAULT = "02";
        /**
         * 设备类型读取
         */
        String CMD_READ_TYPE = "03";
        /**
         * 状态读取 RESISTANCE电阻值
         */
        String CMD_READ_FAULT_RESISTANCE = "03";

        String CMD_ID_GET_SN = "B1";
        int LENGTH_2 = 2;
        int LENGTH_8 = 8;
        int LENGTH_20 = 20;

        /**
         * 请求间隔睡眠时间。
         */
        Long SLEEP = 256L;

        String[] TYPE_1_BIND_MSG = new String[]{"01", "02", "03", "04", "05",
                "06", "07", "08", "09", "0A"};

//        String[] TYPE_1_BIND_MSG = new String[]{"0A", "09", "08", "07", "06",
//                "05"};
        String[] TYPE_2_BIND_MSG = new String[]{"01", "02", "03", "04", "05",
                "06", "07", "08", "09", "0A",
                "0B", "0C"};
        String[] TYPE_3_BIND_MSG = new String[]{"01", "02", "03", "04", "05",
                "06", "07", "08", "09", "0A",
                "0B", "0C", "0D", "0E", "0F"};
    }

    /**
     * 设备类型读取
     *
     * @param simNum sim.sim_num
     */
    public String buildSendMsgReadSimType(final String simNum) {
        return buildSendMsg(simNum, CMD_READ_TYPE, CMD_ID_GET_SN);
    }

    /**
     * 故障下发
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgWriteFault(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_SET_FAULT, bindHardwareMsg);
    }

    /**
     * 状态读取
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgReadFaultResistance(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_READ_FAULT_RESISTANCE, bindHardwareMsg);
    }

    /**
     * 故障清清除
     *
     * @param simNum          sim.sim_num
     * @param bindHardwareMsg fault.bind_hardware_msg
     */
    public String buildSendMsgClearFault(final String simNum, final String bindHardwareMsg) {
        return buildSendMsg(simNum, CMD_CLEAR_FAULT, bindHardwareMsg);
    }

    public String buildSendMsg(final String simNum, final String cmd, final String cmdId) {
        return buildSendMsg(simNum, cmd, cmdId, CMD_DATA_PLACE_HOLDER);
    }

    public String buildSendMsg(final String simNum, final String cmd, final String cmdId, final String data) {
        if (StringUtils.isEmpty(simNum) || StringUtils.isEmpty(cmd) || StringUtils.isEmpty(cmdId) || StringUtils.isEmpty(data)) {
            throw new IllegalArgumentException("buildSendMsg isEmpty");
        }
        if (simNum.length() != LENGTH_2) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        if (cmd.length() != LENGTH_2) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        if (cmdId.length() != LENGTH_2) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        if (data.length() != LENGTH_8) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        StringBuffer m = new StringBuffer();
        m.append(PREFIX);
        m.append(simNum);
        m.append(ORN_SEND);
        m.append(cmd);
        m.append(cmdId);
        m.append(data);
        m.append(SUFFIX);
        final String mFinal = m.toString();
        if (mFinal.length() != LENGTH_20) {
            throw new IllegalArgumentException("buildSendMsg length error");
        }
        return mFinal;
    }

    /**
     * check receiveMsg
     *
     * @param receiveMsg
     * @return
     */
    public boolean checkReceiveMsg(String receiveMsg) {
        if (StringUtils.isEmpty(receiveMsg)) {
            return false;
        }
        if (receiveMsg.length() != LENGTH_20) {
            return false;
        }
        final String orn = StringUtils.substring(receiveMsg, 4, 6);
        if (!ORN_RECEIVE.equals(orn)) {
            return false;
        }
        if (!StringUtils.startsWith(receiveMsg, PREFIX)) {
            return false;
        }
        if (!StringUtils.endsWith(receiveMsg, SUFFIX)) {
            return false;
        }
        return true;
    }
}
