package com.ruoyi.sim.service.impl;

import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.sim.config.SimConfig;
import com.ruoyi.sim.domain.*;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 硬件通信
 * send service.
 */
@Service("commSendService")
public class CommSendService {

    private static final Logger l = LoggerFactory.getLogger(CommSendService.class);
    /**
     * 缓存的Socket连接。
     */
    private Socket cachedSocket = null;
    @Autowired
    private CommReceiveService simReceiveService;
    @Autowired
    private SimService simService;
    @Autowired
    private FaultService faultService;
    @Autowired
    private RealExamService realExamService;
    @Autowired
    private RealExamFaultService realExamFaultService;
    @Autowired
    private SimConfig config;
    @Autowired
    private CommBuildService commBuildService;

    /**
     * 定时任务。
     */
    public void scheduledLoopRead() {
        readAll();
    }

    public void readAll() {
        l.info("readAll");
        List<RealExam> listRE = realExamService.listAllByStatus(RealExam.State.ANSWERING);
        listRE.forEach(e -> {
            if (e == null) {
                return;
            }
            List<RealExamFault> listREF = realExamFaultService.listAllType2LoopReadStateByExamId(e.getExamId());
            listREF.forEach(ref -> {
                Sim s = simService.selectSimBySimId(e.getSimId());
                Fault f = faultService.selectFaultByFaultId(ref.getFaultId());
                if (f != null &&
                        Fault.Type.REAL_GZBW.equals(f.getFaultType()) &&
                        Fault.State.ENABLE.equals(f.getFaultState())
                ) {
                    readOneFaultResistance(s, ref, f, RealExamFault.State.LOOP_READ);
                }
            });
        });
    }

    /**
     * Async version.
     */
    @Async("tp-comm")
    public void readAllAsync() {
        readAll();
    }

    public void readOneExamAtLast(RealExam re) {
        l.info("readOneExamAtLast");
        List<RealExamFault> listREF = realExamFaultService.listAllType2LoopReadStateByExamId(re.getExamId());
        listREF.forEach(ref -> {
            Sim s = simService.selectSimBySimId(re.getSimId());
            Fault f = faultService.selectFaultByFaultId(ref.getFaultId());
            readOneFaultResistance(s, ref, f, RealExamFault.State.FINISH);
        });
        // 计算扣分。
        // 最后都读取到，才算扣分。
        {
            calculateScore(re.getExamId());
        }
    }

    @Async("tp-comm")
    public void readOneExamAtLastAsync(RealExam re) {
        l.info("readOneExamAtLastAsync");
        readOneExamAtLast(re);
    }

    /**
     * 计算减分(不包括超时)。汇总到deduction_total_score字段。
     *
     * @param realExamId
     */
    public void calculateScore(Long realExamId) {
        realExamService.updateOneState(realExamId, RealExam.State.CALCULATING_SCORE);
        int minus = 0;
        // 排除故障部分 + 维修报告选择部分
        realExamFaultService.calculateMinusByRealExamId(realExamId);
        // 超时部分
        int timeoutMinute = 0;
        //
        realExamService.updateOneState(realExamId, RealExam.State.GOT_REPORT);
    }


    /**
     * todo:????
     * 定时任务。
     */
    public void scheduledReadSim() {
        l.info("scheduledReadSim");
        //
        try {
            for (int i = 0; i < 5; i++) {
                debugReadSimType("01");
            }
            debugReadAllFaultResistance("01", "0001");
            debugClearAllFault("01", "0001");
            debugWriteAllFault("01", "0001");
            for (int i = 0; i < 5; i++) {
                debugReadAllFaultResistance("01", "0001");
            }
        } catch (IOException e) {
            throw new RuntimeException(e);
        }

//        if (isSocketOk()) {
//            readAllAsync();
//        } else {
//            scheduledConnect();
//        }
    }

    /**
     * todo:????
     * 定时任务。
     */
    public void scheduledConnect() {
        l.info("scheduledConnect");
        try {
            if (!isReachable(config.getRouterIp())) {
                // ping 不通。
                l.warn("ping not ok");
                simService.updateAllEnableState(Sim.State.GATEWAY_OFFLINE);
            }
            if (!isReachable(config.getRs485Ip())) {
                // todo:ping 不通。
                l.warn("ping not ok");
                simService.updateAllEnableState(Sim.State.GATEWAY_OFFLINE);
            }
            openSocket();
        } catch (IOException e) {
            catchException(e);
        }
        if (isSocketOk()) { // todo:????
            checkAllSimStateAsync();
        }
    }

    /**
     * 查找所有没有被手动禁用，并order by sim_num的模拟器列表。检查所有模拟器状态。
     */
    public void checkAllSimState() {
        List<Sim> list = simService.listAllEnable();
        list.forEach(s -> {
            checkOneSimState(s);
        });
    }

    @Async("tp-comm")
    public void checkAllSimStateAsync() {
        checkAllSimState();
    }

    public void checkOneSimState(Sim s) {
        l.info(s.toString());
        // check todo:
        if (Objects.isNull(s)) {
            return;
        }
        if (Sim.State.DISABLE.equals(s.getSimState())) {
            l.warn("sim DISABLE getSimId = {}", s.getSimId());
            return;
        }
        //
        try {
            SimMsg sm = new SimMsg();
            String sendMsg = commBuildService.buildSendMsgReadSimType(s.getSimNum());
            sm.setSendMsg(sendMsg);
            String receiveMsg = send(sendMsg, s);
            sm.setReceiveMsg(receiveMsg);
            simReceiveService.checkOneSimState(sm, s);
        } catch (IOException e) {
            catchException(e);
        }
    }

    /**
     * static lock
     * todo:
     */
    public static boolean clearListFaultLocked = false;

    /**
     * 清除一个考试的，对应的某型号一台模拟器的，所有设备故障。
     *
     * @param re
     */
    public void clearListFaultByRealExam(RealExam re) {
        // check
        if (Objects.isNull(re)) {

        }
        // about lock
        if (clearListFaultLocked) {
            l.warn("clearListFaultLocked = true");
            return;
        }
        //
        clearListFaultLocked = true;
        {
            realExamService.updateOneState(re.getExamId(), RealExam.State.SIM_WRITING);
        }
        //
        List<RealExamFault> list = realExamFaultService.listAllType2InitStateByExamId(re.getExamId());
        list.forEach(ref -> {
            Fault f = faultService.selectFaultByFaultId(ref.getFaultId());
            if (faultService.isDisable(f.getFaultId())) {
                l.warn("故障{}-Disable", ref.getFaultId());
                return;
            }
            l.info("f.toString() = {}", f);
            Sim s = simService.selectSimBySimId(re.getSimId());
            l.info("s.toString() = {}", s);
            // check
            if (Objects.isNull(f)) {

            }
            clearOneFault(s, ref, f);
        });
        //
        clearListFaultLocked = false;
    }

    /**
     * Async version.
     *
     * @param re
     */
    @Async("tp-comm")
    public void clearListFaultByRealExamAsync(RealExam re) {
        clearListFaultByRealExam(re);
    }

    public void clearAll() {
        // todo:
        simService.listAllEnable().forEach(s -> {
            String simType = s.getSimType();
            List<Fault> listF = faultService.listType3EnableBySimType(simType);
            listF.forEach(f -> {
                clearOneFault(s, null, f);
            });
        });
    }

    public SimMsg debugReadSimType(final String simNum) throws IOException {
        String msg1 = commBuildService.buildSendMsgReadSimType(simNum);
        String msg2 = send(msg1, null);
        return new SimMsg(msg1, msg2);
    }

    public SimMsg debugClearOneFault(final String simNum, final String bindHardwareMsg) throws IOException {
        String msg1 = commBuildService.buildSendMsgClearFault(simNum, bindHardwareMsg);
        String msg2 = send(msg1, null);
        return new SimMsg(msg1, msg2);
    }

    public List<SimMsg> debugClearAllFault(final String simNum, String simType) throws IOException {
        List<SimMsg> list = new ArrayList<>();
        for (String b : CommBuildService.CommConst.TYPE_1_BIND_MSG) {
            list.add(debugClearOneFault(simNum, b));
        }
        return list;
    }

    /**
     * @param s
     * @param reF 可以为空，表示不关联考试的，单独执行的。
     * @param f
     */
    public void clearOneFault(Sim s, RealExamFault reF, Fault f) {
        l.info("清除One故障:getSimNum = {},fault.getName = {}", s.getSimNum(), f.getName());
        // check todo:

        //
        try {
            // step1
            String msg1 = commBuildService.buildSendMsgClearFault(s.getSimNum(), f.getBindHardwareMsg());
            SimMsg sm1 = new SimMsg();
            sm1.setSendMsg(msg1);
            String msg2 = send(msg1, s);
            sm1.setReceiveMsg(msg2);
            simReceiveService.clearOneFault(sm1, s, reF, f);
            // step2
            if (reF != null &&
                    realExamFaultService.isState(reF.getRefId(), RealExamFault.State.CLEARED)) {
                if (reF.getFlag().equals(RealExamFault.Flag.YES)) {
                    writeOneFault(s, reF, f);
                } else if (reF.getFlag().equals(RealExamFault.Flag.NO)) {
                    RealExamFault f1 = realExamFaultService.selectRealExamFaultByRefId(reF.getRefId());
                    f1.setRefState(RealExamFault.State.LOOP_READ);
                }
            }
        } catch (IOException e) {
            catchException(e);
        }
    }

    public SimMsg debugWriteOneFault(final String simNum, final String bindHardwareMsg) throws IOException {
        String msg1 = commBuildService.buildSendMsgWriteFault(simNum, bindHardwareMsg);
        String msg2 = send(msg1, null);
        return new SimMsg(msg1, msg2);
    }

    public List<SimMsg> debugWriteAllFault(final String simNum, String simType) throws IOException {
        List<SimMsg> list = new ArrayList<>();
        for (String b : CommBuildService.CommConst.TYPE_1_BIND_MSG) {
            list.add(debugWriteOneFault(simNum, b));
        }
        return list;
    }

    public void writeOneFault(Sim s, RealExamFault ref, Fault f) {
        l.info("下发故障:getSimId = {},fault.getName = {}", s.getSimId(), f.getName());
        try {
            // todo:ref is null.

            // 下发故障
            SimMsg sm1 = new SimMsg();
            String sendMsg1 = commBuildService.buildSendMsgWriteFault(s.getSimNum(), f.getBindHardwareMsg());
            sm1.setSendMsg(sendMsg1);
            String receiveMsg1 = send(sendMsg1, s);
            sm1.setReceiveMsg(receiveMsg1);
            // todo:

            // 读取一次当前电阻代表值作为出题值。
            SimMsg sm2 = new SimMsg();
            String sendMsg2 = commBuildService.buildSendMsgReadFaultResistance(s.getSimNum(), f.getBindHardwareMsg());
            sm2.setSendMsg(sendMsg2);
            String receiveMsg2 = send(sendMsg2, s);
            sm2.setReceiveMsg(receiveMsg2);
            simReceiveService.setFaultQuestionValue(sm2, s, ref, f);
            if (ref != null) {
                // 修改关联状态。
                RealExamFault f1 = realExamFaultService.selectRealExamFaultByRefId(ref.getRefId());
                f1.setRefState(RealExamFault.State.LOOP_READ);
                realExamFaultService.updateRealExamFault(f1);

                //
                {
                    boolean allWritten = realExamFaultService.isAllType2StateXiaFa(f1.getExamId());
                    if (allWritten) {
                        // 如果全部下发完毕 修改RealExam状态
                        l.info("allWritten id = {}", f1.getExamId());

                    }
                }

                // todo:

                // todo:单独一个故障，特殊处理
                realExamService.updateOneState(f1.getExamId(), RealExam.State.SIM_PREPARE_OK);
            } else {
                l.info("ref is null");
            }
        } catch (IOException e) {
            catchException(e);
        }
    }

    public SimMsg debugReadOneFaultResistance(final String simNum, final String bindHardwareMsg) throws IOException {
        String msg1 = commBuildService.buildSendMsgReadFaultResistance(simNum, bindHardwareMsg);
        String msg2 = send(msg1, null);
        return new SimMsg(msg1, msg2);
    }

    public List<SimMsg> debugReadAllFaultResistance(final String simNum, final String simType) throws IOException {
        List<SimMsg> list = new ArrayList<>();
        for (String b : CommBuildService.CommConst.TYPE_1_BIND_MSG) {
            list.add(debugReadOneFaultResistance(simNum, b));
        }
        return list;
    }

    public void readOneFaultResistance(Sim s, RealExamFault reF, Fault f, String refState) {
        l.info("readOneFaultResistance");
        try {
            SimMsg sm = new SimMsg();
            String sendMsg = commBuildService.buildSendMsgReadFaultResistance(s.getSimNum(), f.getBindHardwareMsg());
            sm.setSendMsg(sendMsg);
            String receiveMsg = send(sendMsg, s);
            sm.setReceiveMsg(receiveMsg);
            simReceiveService.setFaultAnswerValue(sm, s, reF, f, refState);
        } catch (IOException e) {
            catchException(e);
        }
    }

    @Async("tp-comm")
    public void readOneFaultResistanceAsync(Sim s, RealExamFault reF, Fault f, String refState) {
        l.info("readOneFaultResistanceAsync");
        readOneFaultResistance(s, reF, f, refState);
    }

    public void test() {
        try {
            {
                String sendMsg = commBuildService.buildSendMsgReadFaultResistance("01", "04");
                send(sendMsg, null);
            }
            {
                String sendMsg = commBuildService.buildSendMsgReadFaultResistance("01", "05");
                send(sendMsg, null);
            }
            {
                String sendMsg = commBuildService.buildSendMsgReadFaultResistance("01", "06");
                send(sendMsg, null);
            }
        } catch (IOException e) {
            catchException(e);
        }
    }

    /**
     * send hex message
     *
     * @param sendMsg
     * @param s       可以为空，更新最后发送/接收时间 用。
     * @return
     */
    public synchronized String send(final String sendMsg, final Sim s) throws IOException {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == false [模拟器通信被禁用！]");
            return "";
        }
        SimMsg tempSm = new SimMsg();
        // log.
        {
            tempSm.setSendMsg(sendMsg);
            l.info("********sendMsg == [{}]", tempSm);
        }
        //
        {
            try {
                // sleep ,追求顺序请求。
                Thread.sleep(CommBuildService.CommConst.SLEEP);
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
        //
        openSocket();
        String receiveMsg = null;
        InputStream is = cachedSocket.getInputStream();
        OutputStream os = cachedSocket.getOutputStream();
        os.write(hexStrToByteArrs(sendMsg));
        tempSm.setSendTime(DateUtils.getNowDate());
        if (s != null) {
            simService.updateLastSentTime(s);
        }
        byte[] buffer = new byte[1024];
        int length = is.read(buffer);
        StringBuffer sbHex = new StringBuffer();
        for (int i = 0; i < length; i++) {
            sbHex.append(String.format("%02X", buffer[i]));
        }
        receiveMsg = sbHex.toString();
        tempSm.setReceiveTime(DateUtils.getNowDate());
        tempSm.setReceiveMsg(receiveMsg);
        // log.
        {
            l.info("********receiveMsg = [{}]", tempSm);
        }
        if (!commBuildService.checkReceiveMsg(receiveMsg)) {
            // todo:
            l.warn("checkReceiveMsg fail receiveMsg = {}", receiveMsg);
            // closeSocket();
            return "";
        }
        if (s != null) {
            simService.updateLastReceivedTime(s);
        }
        // closeSocket();
        return receiveMsg;
    }

    public void openSocket() throws IOException {
        if (!config.isCommGlobal()) {
            l.warn("isCommGlobal == false [模拟器通信被禁用！]");
            return;
        }
        if (!isSocketOk()) {
            // todo:
            // simService.updateAllEnableState(Sim.State.GATEWAY_OFFLINE);
            l.info("openSocket cachedSocket is null");
            cachedSocket = new Socket(config.getRs485Ip(), config.getRs485Port());
            // setSoTimeout todo:????
            // cachedSocket.setSoTimeout(2000);
        } else {
            l.info("openSocket cachedSocket is ok");
        }
    }

    /**
     * 连接情况是否ok。
     *
     * @return
     */
    public boolean isSocketOk() {
        if (cachedSocket != null && cachedSocket.isConnected()) {
            return true;
        }
        return false;
    }

    /**
     * todo:异常精细处理
     *
     * @throws IOException
     */
    public void closeSocket() throws IOException {
        if (cachedSocket != null) {
            cachedSocket.getInputStream().close();
            cachedSocket.getOutputStream().close();
            cachedSocket.close();
        }
        cachedSocket = null;
    }

    public byte[] hexStrToByteArrs(String hexString) {
//        if (StringUtils.isEmpty(hexString)) {
//            return null;
//        }
        hexString = hexString.replaceAll(" ", "");
        int len = hexString.length();
        int index = 0;

        byte[] bytes = new byte[len / 2];

        while (index < len) {
            String sub = hexString.substring(index, index + 2);
            bytes[index / 2] = (byte) Integer.parseInt(sub, 16);
            index += 2;
        }
        return bytes;
    }

    /**
     * ping
     *
     * @param ipV4
     * @return
     * @throws IOException
     */
    public boolean isReachable(String ipV4) throws IOException {
        InetAddress ia = InetAddress.getByName(ipV4);
        return ia.isReachable(2000);
    }

    private void catchException(IOException e) {
        l.warn("catchException", e);

        // e.printStackTrace();
        // throw new RuntimeException(e);
        {
            clearListFaultLocked = false;
        }
        try {
            closeSocket();
        } catch (IOException ex) {
            throw new RuntimeException(ex);
        }
    }
}
